import waba.ui.*;
import waba.fx.*;
import waba.sys.Time;
import waba.sys.Convert;

/**
  Game thread management
  */
public class GameDrv extends RmblContainer {
  private Graphics grp;
  private HnyObj[] hnyObj;
  private HGEvent[] hgEvt;
  private HGAction[] hgAct;
  private GamePrp gamePrp;
  private int actCnt;
  private HGButton gmTgl;
  private IHenyaG henyaG;
  private HGMsBtn l2Btn, l1Btn, upBtn, downBtn, r1Btn, r2Btn;
  private HGButton stBtn;
  private int score, left;
  Font gmFnt;
  FontMetrics gmFm;
  private int nrnk, ngs;
  private boolean isDemo, isPlayer;
  private int neCnt;
  private int prvSec, prvMin, prvHur;
  private HGCanvas cvs;

  public GameDrv(IHenyaG henyaG, ISurface surf,
   HnyObj[] hnyObj, HGEvent[] hgEvt, GamePrp gamePrp, HGCanvas cvs,
   Graphics grp) {
    this.henyaG = henyaG;
    this.grp = grp;
    this.cvs = cvs;
    this.hnyObj = hnyObj;
    this.hgEvt = hgEvt;
    this.gamePrp = gamePrp;
    gmFnt = new Font("Helvetica", Font.BOLD, 10);
    gmFm = new FontMetrics(gmFnt, surf);

    setRect(0, 0, 160, 160);
    Font fnt = new Font("Helvetica", Font.BOLD, 10);
    stBtn = new HGButton("Start", fnt, 0, 145, 40, 15);

    hgAct = new HGAction[32];
    actCnt = 0;
    isPlayer = false;
  }

  private void startSetup() {
    grp.setDrawOp(Graphics.DRAW_OVER);
    grp.setColor(0, 0, 0);
    cvs.setCanvasParent(null);
    add(cvs);
    cvs.setEnabled(false);
    henyaG.focusThis(this);

    for ( int i=hgAct.length-1 ; i>=0 ; i-- ) {
      hgAct[i] = null;
    }
    int[] onc = new int[99];
    int[] offc = new int[99];
    for ( int i=hgEvt.length-1 ; i>=0 ; i-- ) {
      HGEvent hevt = hgEvt[i];
      if ( hevt != null ) {
        hevt.validData();
        byte[] cnd = hevt.cnd;
        for ( int j=cnd.length-3 ; j>=0 ; j-=3 ) {
          switch ( cnd[j] ) {
            case 1:
              onc[cnd[j+1]]++;
              break;
            case 2:
              offc[cnd[j+1]]++;
              break;
          }
        }
      }
    }
    for ( int i=98 ; i>=0 ; i-- ) {
      if ( hnyObj[i] != null ) {
        if ( onc[i] > 0 ) hnyObj[i].onTrg = new HGEvent[onc[i]];
        if ( offc[i] > 0 ) hnyObj[i].offTrg = new HGEvent[offc[i]];
        hnyObj[i].onTrgCnt = hnyObj[i].offTrgCnt = 0;
      }
    }
    for ( int i=hgEvt.length-1 ; i>=0 ; i-- ) {
      HGEvent hevt = hgEvt[i];
      if ( hevt != null ) {
        byte[] cnd = hevt.cnd;
        for ( int j=cnd.length-3 ; j>=0 ; j-=3 ) {
          switch ( cnd[j] ) {
            case 1:
              hnyObj[cnd[j+1]].addOnTrg(hevt);
              break;
            case 2:
              hnyObj[cnd[j+1]].addOffTrg(hevt);
              break;
          }
        }
      }
    }
    for ( int i=31 ; i>=0 ; i-- ) {
      hgAct[i] = null;
    }
    for ( int i=hnyObj.length-1 ; i>=0 ; i-- ) {
      if ( hnyObj[i] != null ) {
        hnyObj[i].onoff = false;
        HGEvent[] trg = hnyObj[i].offTrg;
        if ( trg != null ) {
          for ( int j = trg.length-1 ; j>=0 ; j-- ) {
            trg[j].onByHnyObj(i);
          }
        }
      }
    }
    cvs.clear();
    if ( cvs.grp != null ) cvs.grp.setColor(255, 255, 255);
  }

  public void start() {
    removeControl();
    startSetup();

    if ( l2Btn == null ) {
      Font fnt = new Font("Helvetica", Font.BOLD, 10);
      gmTgl = new HGButton("Gam", fnt, 0, 0, 25, 15);
      l2Btn = new HGMsBtn("L2", fnt, 2, 145, 26, 15);
      l1Btn = new HGMsBtn("L1", fnt, 28, 145, 26, 15);
      upBtn = new HGMsBtn("Up", fnt, 54, 145, 26, 15);
      downBtn = new HGMsBtn("Dw", fnt, 80, 145, 26, 15);
      r1Btn = new HGMsBtn("R1", fnt, 106, 145, 26, 15);
      r2Btn = new HGMsBtn("R2", fnt, 132, 145, 26, 15);
    }
    add(gmTgl);
    add(l2Btn); add(l1Btn); add(upBtn);
    add(downBtn); add(r1Btn); add(r2Btn);
    score = 0;
    isDemo = false;
    left = gamePrp.left;
    nrnk = gamePrp.rank[0];
    ngs = gamePrp.gameSpeed[0]*2;
    henyaG.setTimerInterval(ngs);
  }

  public void startDemo() {
    isDemo = true;

    removeControl();
    startSetup();
    add(stBtn);

    drawScore();
    neCnt = 0;
    prvSec = prvMin = prvHur = -1;
    nrnk = gamePrp.rank[0];
    ngs = gamePrp.gameSpeed[0]*2;
    henyaG.setTimerInterval(ngs);
  }

  public void setIsPlayer(boolean f) {
    isPlayer = f;
  }

  private void keyPressAction(boolean[] keys) {
    HnyObj.setGrpMode(grp);
    for ( int i=hgEvt.length-1 ; i>=0 ; i-- ) {
      if ( hgEvt[i] != null && hgEvt[i].checkOcr(keys, nrnk) ) {
        for ( int j = 31 ; j >= 0 ; j-- ) {
          actCnt++; actCnt &= 31;
          if ( hgAct[actCnt] == null ) break;
        }
        hgAct[actCnt] = hgEvt[i].getHGAction();
      }
    }
    for ( int i = 31 ; i >= 0 ; i-- ) {
      if ( hgAct[i] != null ) {
        if ( doAction(hgAct[i]) ) hgAct[i] = null;
      }
    }
    HnyObj.resetGrpMode(grp);
  }

  public void onEvent(Event e) {
    if ( e.type == KeyEvent.KEY_PRESS ) {
      KeyEvent ke = (KeyEvent)e;
      int key = ke.key;
      boolean[] keys = new boolean[6];
      switch ( key ) {
        case IKeys.BACKSPACE:
          keys[0] = true;
          break;
        case IKeys.LEFT:
          keys[1] = true;
          break;
        case IKeys.PAGE_UP:
        case IKeys.UP:
          keys[2] = true;
          break;
        case IKeys.PAGE_DOWN:
        case IKeys.DOWN:
          keys[3] = true;
          break;
        case IKeys.RIGHT:
          keys[4] = true;
          break;
        case IKeys.ENTER:
          keys[5] = true;
         break;
      }
      keyPressAction(keys);
    }
    if ( e.type == ControlEvent.PRESSED ) {
      boolean[] keys = new boolean[6];
      boolean kf = false;
      if ( e.target == l2Btn ) { keys[0] = true; kf = true; }
      if ( e.target == l1Btn ) { keys[1] = true; kf = true; }
      if ( e.target == upBtn ) { keys[2] = true; kf = true; }
      if ( e.target == downBtn ) { keys[3] = true; kf = true; }
      if ( e.target == r1Btn ) { keys[4] = true; kf = true; }
      if ( e.target == r2Btn ) { keys[5] = true; kf = true; }
      if ( kf ) keyPressAction(keys);
      if ( e.target == gmTgl ) {
        remove(cvs);
        henyaG.changeMode(0);
      }
      if ( e.target == stBtn ) {
        remove(cvs);
        henyaG.changeMode(4);
      }
    }
  }

  public void onPaint(Graphics g) {
    if ( grp == null ) return;

    HnyObj.resetGrpMode(grp);
    grp.setColor(255, 255, 255);
    grp.fillRect(0, 0, 160, 160);
    if ( isDemo ) {
      prvSec = -1;
      drawTime(new Time());
      drawScore();
    } else {
      drawScore();
      drawLeft();
    }
    HnyObj.setGrpMode(grp);
    for ( int i=98 ; i>=0 ; i-- ) {
      if ( hnyObj[i] != null && gamePrp.dspCtg[hnyObj[i].ctg] && hnyObj[i].onoff ) {
        hnyObj[i].draw();
      }
    }
  }

  private void drawScore() {
    HnyObj.resetGrpMode(grp);
    grp.setColor(255, 255, 255);
    grp.fillRect(100, 0, 60, 15);
    grp.setColor(0, 0, 0);
    String scr = Convert.toString(score);
    int textX = gmFm.getTextWidth(scr);
    grp.drawText(scr, 156-textX, 3);
    HnyObj.setGrpMode(grp);
  }

  private void drawLeft() {
    HnyObj.resetGrpMode(grp);
    grp.setColor(255, 255, 255);
    grp.fillRect(50, 0, 40, 15);
    grp.setColor(0, 0, 0);
    grp.drawText(Convert.toString(left), 60, 3);
    HnyObj.setGrpMode(grp);
  }

  private void drawTime(Time tm) {
    HnyObj.resetGrpMode(grp);
    int sc = tm.second;
    int mn = tm.minute;
    int hr = tm.hour;
    char[] s = new char[1];
    grp.setColor(255, 255, 255);
    if ( prvSec == -1 || prvMin != mn || prvHur != hr ) {
      grp.fillRect(100, 145, 60, 15);
      grp.setColor(0, 0, 0);
      s[0] = (char)('0' + (int)(hr/10));
      grp.drawText(new String(s), 100, 148);
      s[0] = (char)('0' + (hr%10));
      grp.drawText(new String(s), 108, 148);
      grp.drawText(":", 116, 148);
      s[0] = (char)('0' + (int)(mn/10));
      grp.drawText(new String(s), 120, 148);
      s[0] = (char)('0' + (mn%10));
      grp.drawText(new String(s), 128, 148);
      grp.drawText(":", 136, 148);
      s[0] = (char)('0' + (int)(sc/10));
      grp.drawText(new String(s), 140, 148);
      s[0] = (char)('0' + (sc%10));
      grp.drawText(new String(s), 148, 148);
      prvMin = mn; prvHur = hr;
    } else {
      grp.fillRect(140, 145, 20, 15);
      grp.setColor(0, 0, 0);
      s[0] = (char)('0' + (int)(sc/10));
      grp.drawText(new String(s), 140, 148);
      s[0] = (char)('0' + (sc%10));
      grp.drawText(new String(s), 148, 148);
    }
    prvSec = sc;
    HnyObj.setGrpMode(grp);
  }

  private boolean turnOnOff(int idx, boolean f) {
    if ( hnyObj[idx].onoff != f ) {
      HnyObj hobj = hnyObj[idx];
      hobj.onoff = f;
      if ( gamePrp.dspCtg[hobj.ctg] ) {
        if ( f ) {
          hobj.draw();
        } else {
          hobj.clear();
        }
      }
      HGEvent[] trg = hobj.onTrg;
      if ( trg != null ) {
        if ( f ) {
          for ( int i = trg.length-1 ; i>=0 ; i-- ) {
            trg[i].onByHnyObj(idx);
          }
        } else {
          for ( int i = trg.length-1 ; i>=0 ; i-- ) {
            trg[i].offByHnyObj(idx);
          }
        }
      }
      trg = hobj.offTrg;
      if ( trg != null ) {
        if ( f ) {
          for ( int i = trg.length-1 ; i>=0 ; i-- ) {
            trg[i].offByHnyObj(idx);
          }
        } else {
           for ( int i = trg.length-1 ; i>=0 ; i-- ) {
            trg[i].onByHnyObj(idx);
          }
        }
      }
      return true;
    }
    return false;
  }

  private void addScore(int ads) {
    score += ads;
    if ( score > 999 ) score = 999;
    int pgs = ngs;
    nrnk = gamePrp.rank[(int)(score/20)];
    ngs = gamePrp.gameSpeed[(int)(score/20)]*2;
    if ( ngs != pgs ) henyaG.setTimerInterval(ngs);
  }

  private boolean doAction(HGAction hAct) {
    while ( hAct.timer <= 0 ) {
      int cnt = hAct.actCnt;
      byte[] act = hAct.act;
      if ( cnt >= 3 && act[cnt-3] == 3 ) {
        if ( !turnOnOff(act[cnt-2], false) ) {
          hAct.actCnt = act.length;
          act[cnt] = -1;
        }
      }
      switch ( act[cnt] ) {
        case 1:
          turnOnOff(act[cnt+1], true);
          break;
        case 2:
          turnOnOff(act[cnt+1], false);
          break;
        case 3:
          turnOnOff(act[cnt+1], true);
          break;
        case 4:
          if ( isDemo ) break;
          left--;
          drawLeft();
          if ( left <= 0 ) {
            if ( isPlayer ) henyaG.changeMode(5);
            else henyaG.setTimerInterval(0);
          }
          break;
        case 5:
          if ( isDemo ) break;
          addScore(act[cnt+1]);
          drawScore();
          break;
      }
      hAct.actCnt += 3;
      if ( hAct.actCnt >= act.length ) {
        return true;
      } else {
        hAct.timer = act[hAct.actCnt+2];
      }
    }
    return false;
  }

  private int abs(int n) {
    if ( n >= 0 ) return n;
    else return -n;
  }

  public void onTimerTick() {
    HnyObj.setGrpMode(grp);
    for ( int i = hgEvt.length-1 ; i >= 0 ; i-- ) {
      if ( hgEvt[i] != null && hgEvt[i].checkOcr(nrnk) ) {
        for ( int j = 31 ; j >= 0 ; j-- ) {
          actCnt++; actCnt &= 31;
          if ( hgAct[actCnt] == null ) break;
        }
        hgAct[actCnt] = hgEvt[i].getHGAction();
        neCnt = 0;
      }
    }
    if ( isDemo ) {
      HnyObj.resetGrpMode(grp);
      Time tm = new Time();
      if ( tm.second != prvSec ) {
        drawTime(tm);
      }
      neCnt++;
      if ( neCnt > 48 ) {
        henyaG.changeMode(5);
        return;
      }
      if ( (Rnd.nextInt()&7) == 0 ) {
        boolean[] keys = new boolean[6];
        keys[(int)(abs(Rnd.nextInt()%12)/2)] = true;
        keyPressAction(keys);
      }
      HnyObj.setGrpMode(grp);
    }
    for ( int i = 31 ; i >= 0 ; i-- ) {
      if ( hgAct[i] != null ) {
        hgAct[i].timer--;
        if ( doAction(hgAct[i]) ) hgAct[i] = null;
      }
    }
    HnyObj.resetGrpMode(grp);
  }
}

