import waba.ui.*;
import waba.fx.*;

/**
  Screen canvas class
  */
public class HGCanvas extends Control {
  public Image oimg;
  public Graphics oig, grp;
  private int lx, ly;
  private boolean inPress;
  private boolean enabled;
  private byte[] pdx, pdy;
  private int pdIdx;

  public HGCanvas(int x, int y, int width, int height) {
    setRect(x, y, width, height);
    inPress = false;
    lx = -1;
    enabled = true;
    oimg = new Image(160, 130);
    oig = new Graphics(oimg);
    oig.setDrawOp(Graphics.DRAW_OVER);
    clearObjImage();
    setImage();
    pdx = new byte[256]; pdy = new byte[256];
    pdIdx = 0;
  }

  public void setEnabled(boolean f) {
    enabled = f;
  }

  public void setImage() {
    pdIdx = 0;
  }

  public void setGraphics() {
    grp = createGraphics();
    grp.setDrawOp(Graphics.DRAW_OVER);
  }

  public void clearObjImage() {
    oig.setColor(255, 255, 255);
    oig.fillRect(0, 0, 160, 130);
    oig.setColor(0, 0, 0);
  }

  public void clear() {
    grp.setColor(255, 255, 255);
    grp.fillRect(0, 0, 160, 130);
    grp.setColor(0, 0, 0);
  }

  public HnyObj getHObj() {
    if ( pdIdx <= 0 ) return null;
    if ( grp == null ) repaint();
    HnyObj hobj = new HnyObj(grp, oimg, pdx, pdy, pdIdx);
    return hobj;
  }

  public void onEvent(Event e) {
    if ( e.type == PenEvent.PEN_DOWN ) {
      if ( enabled ) inPress = true;
      else return;
      PenEvent pe = (PenEvent)e;
      if ( pdIdx >= 256 ) return;
      pdx[pdIdx] = -1;
      pdIdx++; if ( pdIdx >= 256 ) return;
      pdx[pdIdx] = (byte)pe.x; pdy[pdIdx] = (byte)pe.y;
      pdIdx++;
      lx = pe.x; ly = pe.y;
    } else if ( e.type == PenEvent.PEN_DRAG || e.type == PenEvent.PEN_UP ) {
      if ( inPress ) {
        PenEvent pe = (PenEvent)e;
        if ( pe.x<0 || pe.x>=width || pe.y<0 || pe.y>=height ) {
          lx = -1;
          return;
        }
        if ( pdIdx >= 256 ) return;
        if ( lx != -1 ) {
          grp.drawLine(lx, ly, pe.x, pe.y);
        } else {
          pdx[pdIdx] = -1;
          pdIdx++; if ( pdIdx >= 256 ) return;
        }
        pdx[pdIdx] = (byte)pe.x; pdy[pdIdx] = (byte)pe.y;
        pdIdx++;
        lx = pe.x; ly = pe.y;
      } else {
        lx = -1;
      }
      if ( e.type == PenEvent.PEN_UP ) {
        inPress = false;
        lx = -1;
      }
    }
  }

  public void drawCurrentHnyObj() {
    if ( pdIdx < 1 ) return;
    int px = pdx[pdIdx-1]&255, py = pdy[pdIdx-1]&255, npx, npy;
    for ( int i = pdIdx-2 ; i > 0 ; i-- ) {
      if ( pdx[i] != -1 ) {
        npx = pdx[i]&255; npy = pdy[i]&255;
        grp.drawLine(px, py, npx, npy);
      } else {
        i--;
        npx = pdx[i]&255; npy = pdy[i]&255;
      }
      px = npx; py = npy;
    }
  }

  public void blinkHnyObj(HnyObj hobj, int blinkCnt) {
      if ( (blinkCnt&1)==0 ) grp.setColor(0, 0, 0);
      else grp.setColor(255, 255, 255);
      if ( hobj.y >= 0 ) {
        grp.drawLine(hobj.x-1, hobj.y-1, hobj.x+hobj.width, hobj.y-1);
      }
      grp.drawLine(hobj.x+hobj.width, hobj.y-1, hobj.x+hobj.width, hobj.y+hobj.height);
      if ( hobj.y+hobj.height < height ) {
        grp.drawLine(hobj.x+hobj.width, hobj.y+hobj.height, hobj.x-1, hobj.y+hobj.height);
      }
      grp.drawLine(hobj.x-1, hobj.y+hobj.height, hobj.x-1, hobj.y-1);
      grp.setColor(0, 0, 0);
  }

  public void drawOnOff(HnyObj hobj, int spc) {
    drawOnOffOfs(hobj, spc, 0, 0);
  }

  public void drawOnOff2(HnyObj hobj, int spc) {
    drawOnOffOfs(hobj, spc, hobj.width-5, hobj.height-5);
  }

  public void clearOnOff(HnyObj hobj) {
    grp.setColor(255, 255, 255);
    grp.fillRect(hobj.x, hobj.y, 5, 5);
    grp.fillRect(hobj.x+hobj.width-5, hobj.y+hobj.height-5, 5, 5);
    grp.setColor(0, 0, 0);
  }

  public void drawOnOffOfs(HnyObj hobj, int spc, int ofx, int ofy) {
    int hx = hobj.x + ofx, hy = hobj.y + ofy;
    grp.setColor(255, 255, 255);
    grp.fillRect(hx, hy, 5, 5);
    grp.setColor(0, 0, 0);
    switch (spc) {
      case 1:
        grp.fillRect(hx, hy, 5, 5);
        break;
      case 2:
        grp.drawLine(hx, hy, hx+4, hy);
        grp.drawLine(hx+4, hy, hx+4, hy+4);
        grp.drawLine(hx+4, hy+4, hx, hy+4);
        grp.drawLine(hx, hy+4, hx, hy);
        break;
      case 3:
        grp.drawLine(hx, hy, hx+4, hy+4);
        grp.drawLine(hx+4, hy, hx, hy+4);
        break;
    }
  }

  private ICanvasParent cvsParent;

  public void setCanvasParent(ICanvasParent cp) {
    cvsParent = cp;
  }

  public void onPaint(Graphics g) {
    if ( cvsParent != null ) {
      cvsParent.canvasPainted(g);
    }
  }
}
