import waba.ui.*;
import waba.fx.*;
import waba.sys.Convert;

/**
  ToggleBox UI class
  */
public class HGToggleBox extends Control {
  static final int CHECK = 0;
  static final int TOGGLE = 1;
  static final int BUTTON = 2;
  static final int CHECKEDBUTTON = 3;

  private Image img;
  private Graphics ig;
  private int bwd, bhg, column, line;
  public boolean[] check;
  private Font font;
  private FontMetrics fm;
  private boolean tglf;
  public int tglIdx;
  private boolean btnf;
  private boolean chkf;

  public HGToggleBox(String[] texts, Font font, ISurface surf, int column, int line,
   int type,
   int x, int y, int width, int height) {
    int wd = width*column+1, hg = height*line+1;
    setRect(x, y, wd, hg);
    switch ( type ) {
      case CHECK:
        tglf = false; btnf = false; chkf = true; break;
      case TOGGLE:
        tglf = true; btnf = false; chkf = true; break;
      case BUTTON:
        tglf = true; btnf = true; chkf = true; break;
      case CHECKEDBUTTON:
        tglf = true; btnf = true; chkf = false; break;
    }
    bwd = width; bhg = height;
    this.column = column; this.line = line;
    this.font = font;
    String[] text = texts;
    check = new boolean[column*line];
    tglIdx = -1;

    int stn = -9999;
    if ( text == null ) {
      stn = 1;
    } else if ( text.length==1 && ( column>1  || line>1 ) ) {
      stn = Convert.toInt(text[0]);
    }
    if ( stn != -9999 ) {
      text = new String[column*line];
      for ( int i=0 ; i<column*line ; i++ ) {
        text[i] = Convert.toString(i+stn);
      }
    }

    img = new Image(wd, hg);
    ig = new Graphics(img);
    ig.setColor(255, 255, 255);
    ig.fillRect(0, 0, wd, hg);
    ig.setColor(0, 0, 0);
    ig.setFont(font);
    for ( int lx=0 ; lx<wd ; lx+=width ) {
      ig.drawLine(lx, 0, lx, hg);
    }
    for ( int ly=0 ; ly<hg ; ly+=height ) {
      ig.drawLine(0, ly, wd, ly);
    }

    int cnt = 0;
    int textX, textY;
    fm = new FontMetrics(font, surf);
    textY = (height - fm.getHeight()) / 2;
    for ( int by=0 ; by<line ; by++ ) {
      for ( int bx=0 ; bx<column ; bx++ ) {
        textX = (width - fm.getTextWidth(text[cnt])) / 2;
        ig.drawText(text[cnt], bx*width+textX+1, by*height+textY+1);
        cnt++;
      }
    }
  }

  private void checkBox(int idx) {
    int bx = (idx%column)*bwd, by = idx/column*bhg;
    ig.drawLine(bx+2, by+2, bx+bwd-2, by+2);
    ig.drawLine(bx+bwd-2, by+2, bx+bwd-2, by+bhg-2);
    ig.drawLine(bx+bwd-2, by+bhg-2, bx+2, by+bhg-2);
    ig.drawLine(bx+2, by+bhg-2, bx+2, by+2);
  }

  private void uncheckBox(int idx) {
    int bx = (idx%column)*bwd, by = idx/column*bhg;
    ig.setColor(255, 255, 255);
    ig.drawLine(bx+2, by+2, bx+bwd-2, by+2);
    ig.drawLine(bx+bwd-2, by+2, bx+bwd-2, by+bhg-2);
    ig.drawLine(bx+bwd-2, by+bhg-2, bx+2, by+bhg-2);
    ig.drawLine(bx+2, by+bhg-2, bx+2, by+2);
    ig.setColor(0, 0, 0);
  }

  public void changeCheck(boolean f, int idx) {
    if ( check[idx] == f ) return;
    check[idx] = f;
    if ( f ) checkBox(idx);
    else uncheckBox(idx);
  }

  public void setTglIdx(int idx) {
    if ( chkf ) {
      if ( tglIdx!=-1 ) {
        uncheckBox(tglIdx);
      }
      checkBox(idx);
    }
    tglIdx = idx;
    repaint();
  }

  public void clearCheck() {
    for ( int i=check.length-1 ; i>=0 ; i-- ) {
      check[i] = false;
      uncheckBox(i);
    }
    tglIdx = -1;
    repaint();
  }

  public void onEvent(Event e) {
    if ( e.type == PenEvent.PEN_DOWN ) {
      PenEvent pe = (PenEvent)e;
      if ( (pe.x%bwd)==0 || (pe.y%bhg)==0 ) return;
      int idx = pe.x/bwd+pe.y/bhg*column;
      if ( tglf ) {
        setTglIdx(idx);
        parent.postEvent(new HGTglBoxEvent(HGTglBoxEvent.IDXCHANGED, this, idx));
      } else {
        if ( check[idx] ) {
          uncheckBox(idx);
          check[idx] = false;
          parent.postEvent(new HGTglBoxEvent(HGTglBoxEvent.UNCHECKED, this, idx));
        } else {
          checkBox(idx);
          check[idx] = true;
          parent.postEvent(new HGTglBoxEvent(HGTglBoxEvent.CHECKED, this, idx));
        }
      }
      repaint();
    } else if ( e.type == PenEvent.PEN_UP ) {
      if ( btnf && tglIdx!=-1 ) {
        PenEvent pe = (PenEvent)e;
        if ( chkf ) uncheckBox(tglIdx);
        repaint();
        if ( (pe.x%bwd)==0 || (pe.y%bhg)==0 ) return;
        int idx = pe.x/bwd+pe.y/bhg*column;
        if ( idx == tglIdx ) {
          parent.postEvent(new HGTglBoxEvent(HGTglBoxEvent.PRESSED, this, tglIdx));
        }
      }
    }
  }

  public void onPaint(Graphics grp) {
    grp.drawImage(img, 0, 0);
  }
}
