import waba.ui.*;
import waba.fx.*;
import waba.io.*;

/**
  HenyaG main class
  */
public class HenyaG extends MainWindow implements IHenyaG {
  final static private int DURATION = 100;
  private MainMenu mainMenu;
  private InputObj inputObj;
  private InputEvt inputEvt;
  private InputPrp inputPrp;
  private GameDrv gameDrv;
  private int mode;

  private HGCanvas cvs;

  private HnyObj[] hnyObj;
  private HGEvent[] hgEvt;
  private int slotNmb;
  private String hoCtrStr, heCtrStr, gpCtrStr;
  private GamePrp gamePrp;

  private boolean clgf = false;
  private String message;

  private Timer timer;
  private Graphics grp;

  public HenyaG() {
    hnyObj = new HnyObj[99];
    for ( int i=98 ; i>=0 ; i-- ) hnyObj[i] = null;
    hgEvt = new HGEvent[99];
    for ( int i=98 ; i>=0 ; i-- ) hgEvt[i] = null;
    gamePrp = new GamePrp();
  }

  public void onPaint(Graphics g) {
    if ( grp == null ) {
      grp = new Graphics(this);
      cvs = new HGCanvas(0, 15, 160, 130);
      mainMenu = new MainMenu(this, this);
      inputObj = new InputObj(this, this, hnyObj, hgEvt, cvs);
      inputEvt = new InputEvt(this, this, hnyObj, hgEvt, cvs);
      inputPrp = new InputPrp(this, this, gamePrp);
      gameDrv = new GameDrv(this, this, hnyObj, hgEvt, gamePrp, cvs, grp);
      slotNmb = -1;
      changeMode(0);
    }
  }

  private void removeControl() {
    while ( null != children ) {
      remove(children);
    }
  }

  public void changeMode(int mode) {
    if ( slotNmb==-1 && mode>0 ) return;
    removeControl();
    if ( timer != null ) {
      removeTimer(timer);
    }
    this.mode = mode;
    grp.setDrawOp(Graphics.DRAW_OVER);
    grp.clearClip();
    grp.setColor(0, 0, 0);
    switch ( mode ) {
      case 0:
        add(mainMenu);
        break;
      case 1:
        add(inputObj);
        inputObj.start();
        timer = addTimer(DURATION);
        break;
      case 2:
        add(inputEvt);
        inputEvt.start();
        break;
      case 3:
        add(inputPrp);
        inputPrp.start();
        break;
      case 4:
        add(gameDrv);
        gameDrv.start();
        break;
    }
  }

  public void drawMessage(String msg) {
    drawMessage(createGraphics(), msg);
  }

  private void drawMessage(Graphics grp, String msg) {
    grp.setColor(255, 255, 255);
    grp.fillRect(40, 60, 80, 40);
    grp.setColor(0, 0, 0);
    grp.drawLine(40, 60, 120, 60);
    grp.drawLine(120, 60, 120, 100);
    grp.drawLine(120, 100, 40, 100);
    grp.drawLine(40, 100, 40, 60);
    grp.drawText(msg, 45, 70);
  }

  public void setTimerInterval(int time) {
    if ( timer != null ) removeTimer(timer);
    if ( time > 0 ) timer = addTimer(this, time);
  }

  public void focusThis(Control cnt) {
    setFocus(cnt);
  }

  public void onEvent(Event e) {
    if ( e.type == ControlEvent.TIMER ) {
      switch ( mode ) {
        case 1:
          inputObj.onTimerTick();
          break;
        case 4:
          gameDrv.onTimerTick();
          break;
      }
    }
  }

  public void loadHnyObj() {
    Catalog ct = new Catalog(hoCtrStr, Catalog.READ_ONLY);
    if ( !ct.isOpen() ) return;
    for ( int i=ct.getRecordCount()-1 ; i>=0 ; i-- ) {
      ct.setRecordPos(i);
      int lng = ct.getRecordSize();
      byte[] bst = new byte[lng];
      ct.readBytes(bst, 0, lng);
      int hIdx = bst[0] + bst[1]*128;
      hnyObj[hIdx] = new HnyObj(cvs.grp, cvs.oimg);
      hnyObj[hIdx].readObject(bst);

      hnyObj[hIdx].draw(cvs.oig);
    }
    ct.close();
  }

  public void loadHGEvent() {
    Catalog ct = new Catalog(heCtrStr, Catalog.READ_ONLY);
    if ( !ct.isOpen() ) return;
    for ( int i=ct.getRecordCount()-1 ; i>=0 ; i-- ) {
      ct.setRecordPos(i);
      int lng = ct.getRecordSize();
      byte[] bst = new byte[lng];
      ct.readBytes(bst, 0, lng);

      byte[] ost = new byte[lng-2];
      int eIdx = bst[0] + bst[1]*128;
      hgEvt[eIdx] = new HGEvent();
      hgEvt[eIdx].readObject(bst);
    }
    ct.close();
  }

  public void loadGamePrp() {
    Catalog ct = new Catalog(gpCtrStr, Catalog.READ_ONLY);
    if ( !ct.isOpen() ) return;
    ct.setRecordPos(0);
    int lng = ct.getRecordSize();
    byte[] bst = new byte[lng];
    ct.readBytes(bst, 0, lng);
    gamePrp.readObject(bst);
    ct.close();
  }

  public void loadSlot(int nmb) {
    if ( cvs.grp == null ) {
      add(cvs);
      cvs.repaint();
      cvs.setGraphics();
      remove(cvs);
    }
    clearAll();
    slotNmb = nmb;
    char[] nc = {'a', 'a', 'n', 'T', '.', 'H', 'O', '0', '0'};
    nc[7] = (char)('0' + (int)(nmb/10));
    nc[8] = (char)('0' + (int)(nmb%10));
    hoCtrStr = new String(nc);
    nc[6] = 'E';
    heCtrStr = new String(nc);
    nc[5] = 'G';
    nc[6] = 'P';
    gpCtrStr = new String(nc);
    loadHnyObj();
    for ( int i=0 ; i<99 ; i++ ) {
      if ( hnyObj[i] == null ) {
        inputObj.objNmb.setNumber(i+1);
        break;
      }
    }
    loadHGEvent();
    loadGamePrp();
  }

  public void clearAll() {
    cvs.clearObjImage();
    for ( int i=98 ; i>=0 ; i-- ) hnyObj[i] = null;
    for ( int i=98 ; i>=0 ; i-- ) hgEvt[i] = null;
    inputObj.objNmb.setNumber(1);
    gamePrp.init();
  }

  public void clearDB() {
    if ( slotNmb == -1 ) return;
    Catalog ct = new Catalog(hoCtrStr, Catalog.WRITE_ONLY);
    if ( ct.isOpen() ) {
      ct.delete();
      ct.close();
    }
    ct = new Catalog(heCtrStr, Catalog.WRITE_ONLY);
    if ( ct.isOpen() ) {
      ct.delete();
      ct.close();
    }
    ct = new Catalog(gpCtrStr, Catalog.WRITE_ONLY);
    if ( ct.isOpen() ) {
      ct.delete();
      ct.close();
    }
  }

  public void saveHnyObj() {
    Catalog ct = new Catalog(hoCtrStr, Catalog.CREATE);
    if ( !ct.isOpen() ) return;
    for ( int i=98 ; i>=0 ; i-- ) {
      if ( hnyObj[i] != null ) {
        byte[] bst = hnyObj[i].writeObject();
        bst[0] = (byte)(i&127); bst[1] = (byte)(i/128);
        ct.addRecord(bst.length);
        ct.writeBytes(bst, 0, bst.length);
        hnyObj[i] = null;
      }
    }
    ct.close();
    return;
  }

  public void saveHGEvent() {
    Catalog ct = new Catalog(heCtrStr, Catalog.CREATE);
    if ( !ct.isOpen() ) return;
    for ( int i=98 ; i>=0 ; i-- ) {
      if ( hgEvt[i] != null && hgEvt[i].cndCnt > 0 && hgEvt[i].actCnt > 0 ) {
        byte[] bst = hgEvt[i].writeObject();
        bst[0] = (byte)(i&127); bst[1] = (byte)(i/128);
        ct.addRecord(bst.length);
        ct.writeBytes(bst, 0, bst.length);
      }
    }
    ct.close();
  }

  public void saveGamePrp() {
    Catalog ct = new Catalog(gpCtrStr, Catalog.CREATE);
    if ( !ct.isOpen() ) return;
    byte[] bst = gamePrp.writeObject();
    ct.addRecord(bst.length);
    ct.writeBytes(bst, 0, bst.length);
    ct.close();
  }

  public void saveSlot() {
    if ( slotNmb == -1 ) return;
    clearDB();
    saveHnyObj();
    saveHGEvent();
    saveGamePrp();
  }

  public void onExit() {
    saveSlot();
  }
}
